@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2011, 2014, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -486,35 +486,40 @@ def show_server_info(servers, options):
             server1 = Server(options={'conn_info': conn_dict})
             server_is_off = False
             # If we got errno 2002 it means can not connect through the
-            # given socket, but if path to socket not empty, server could be
-            # turned off.
+            # given socket.
             if util_error.errno == CR_CONNECTION_ERROR:
                 socket = conn_dict.get("unix_socket", "")
                 if socket:
-                    mydir = os.path.split(socket)[0]
-                    if os.path.isdir(mydir) and len(os.listdir(mydir)) != 0:
-                        server_is_off = True
-            # If we got errno 2003 and this is a windows, we do not have
+                    msg = ("Unable to connect to server using socket "
+                           "'{0}'.".format(socket))
+                    if os.path.isfile(socket):
+                        err_msg = ["{0} Socket file is not valid.".format(msg)]
+                    else:
+                        err_msg = ["{0} Socket file does not "
+                                   "exist.".format(msg)]
+            # If we got errno 2003 and we do not have
             # socket, instead we check if server is localhost.
             elif (util_error.errno == CR_CONN_HOST_ERROR and
-                  os.name == 'nt' and server1.is_alias("localhost")):
+                    server1.is_alias("localhost")):
                 server_is_off = True
             # If we got errno 1045 it means Access denied,
             # notify the user if a password was used or not.
             elif util_error.errno == ER_ACCESS_DENIED_ERROR:
                 use_pass = 'YES' if conn_dict['passwd'] else 'NO'
-                er = ("Access denied for user '{0}'@'{1}' using password: {2}"
-                      ).format(conn_dict['user'], conn_dict['host'], use_pass)
+                err_msg = ("Access denied for user '{0}'@'{1}' using "
+                           "password: {2}".format(conn_dict['user'],
+                                                  conn_dict['host'],
+                                                  use_pass))
             # Use the error message from the connection attempt.
             else:
-                er = [util_error.errmsg]
+                err_msg = [util_error.errmsg]
             # To propose to start a cloned server for extract the info,
             # can not predict if the server is really off, but we can do it
             # in case of socket error, or if one of the related
             # parameter was given.
-            if (server_is_off or basedir or datadir or start):
-                er = ["Server is offline. To connect, "
-                      "you must also provide "]
+            if server_is_off or basedir or datadir or start:
+                err_msg = ["Server is offline. To connect, "
+                           "you must also provide "]
 
                 opts = ["basedir", "datadir", "start"]
                 for opt in tuple(opts):
@@ -524,15 +529,15 @@ def show_server_info(servers, options):
                     except KeyError:
                         pass
                 if opts:
-                    er.append(", ".join(opts[0:-1]))
+                    err_msg.append(", ".join(opts[0:-1]))
                     if len(opts) > 1:
-                        er.append(" and the ")
-                    er.append(opts[-1])
-                    er.append(" option")
-                    raise UtilError("".join(er))
+                        err_msg.append(" and the ")
+                    err_msg.append(opts[-1])
+                    err_msg.append(" option")
+                    raise UtilError("".join(err_msg))
 
             if not start:
-                raise UtilError("".join(er))
+                raise UtilError("".join(err_msg))
             else:
                 try:
                     server_val = parse_connection(server, None, options)
